/*******************************************************************************
 * Copyright © 2011, 2012 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM Corporation - initial API and implementation
 *
 *******************************************************************************/
package org.eclipse.edt.ide.ui.internal.deployment.provider;


import java.util.Collection;
import java.util.List;

import org.eclipse.edt.ide.ui.internal.deployment.Deployment;
import org.eclipse.edt.ide.ui.internal.deployment.DeploymentFactory;
import org.eclipse.edt.ide.ui.internal.deployment.DeploymentPackage;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;

import org.eclipse.emf.common.util.ResourceLocator;

import org.eclipse.emf.ecore.EStructuralFeature;

import org.eclipse.emf.ecore.util.FeatureMapUtil;

import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ItemProviderAdapter;
import org.eclipse.emf.edit.provider.ViewerNotification;

/**
 * This is the item provider adapter for a {@link org.eclipse.edt.ide.ui.internal.deployment.Deployment} object.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class DeploymentItemProvider
	extends ItemProviderAdapter
	implements
		IEditingDomainItemProvider,
		IStructuredItemContentProvider,
		ITreeItemContentProvider,
		IItemLabelProvider,
		IItemPropertySource
{
	/**
	 * This constructs an instance from a factory and a notifier.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public DeploymentItemProvider(AdapterFactory adapterFactory)
	{
		super(adapterFactory);
	}

	/**
	 * This returns the property descriptors for the adapted class.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object)
	{
		if (itemPropertyDescriptors == null)
		{
			super.getPropertyDescriptors(object);

		}
		return itemPropertyDescriptors;
	}

	/**
	 * This specifies how to implement {@link #getChildren} and is used to deduce an appropriate feature for an
	 * {@link org.eclipse.emf.edit.command.AddCommand}, {@link org.eclipse.emf.edit.command.RemoveCommand} or
	 * {@link org.eclipse.emf.edit.command.MoveCommand} in {@link #createCommand}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Collection<? extends EStructuralFeature> getChildrenFeatures(Object object)
	{
		if (childrenFeatures == null)
		{
			super.getChildrenFeatures(object);
			childrenFeatures.add(DeploymentPackage.Literals.DEPLOYMENT__BINDINGS);
			childrenFeatures.add(DeploymentPackage.Literals.DEPLOYMENT__SERVICES);
			childrenFeatures.add(DeploymentPackage.Literals.DEPLOYMENT__RUIAPPLICATION);
			childrenFeatures.add(DeploymentPackage.Literals.DEPLOYMENT__RESOURCE_OMISSIONS);
			childrenFeatures.add(DeploymentPackage.Literals.DEPLOYMENT__TARGET_GROUP);
			childrenFeatures.add(DeploymentPackage.Literals.DEPLOYMENT__INCLUDE);
			childrenFeatures.add(DeploymentPackage.Literals.DEPLOYMENT__DEPLOY_EXT_GROUP);
		}
		return childrenFeatures;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EStructuralFeature getChildFeature(Object object, Object child)
	{
		// Check the type of the specified child object and return the proper feature to use for
		// adding (see {@link AddCommand}) it as a child.

		return super.getChildFeature(object, child);
	}

	/**
	 * This returns Deployment.gif.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object getImage(Object object)
	{
		return overlayImage(object, getResourceLocator().getImage("full/obj16/Deployment"));
	}

	/**
	 * This returns the label text for the adapted class.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public String getText(Object object)
	{
		return getString("_UI_Deployment_type");
	}

	/**
	 * This handles model notifications by calling {@link #updateChildren} to update any cached
	 * children and by creating a viewer notification, which it passes to {@link #fireNotifyChanged}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void notifyChanged(Notification notification)
	{
		updateChildren(notification);

		switch (notification.getFeatureID(Deployment.class))
		{
			case DeploymentPackage.DEPLOYMENT__BINDINGS:
			case DeploymentPackage.DEPLOYMENT__SERVICES:
			case DeploymentPackage.DEPLOYMENT__RUIAPPLICATION:
			case DeploymentPackage.DEPLOYMENT__RESOURCE_OMISSIONS:
			case DeploymentPackage.DEPLOYMENT__TARGET_GROUP:
			case DeploymentPackage.DEPLOYMENT__INCLUDE:
			case DeploymentPackage.DEPLOYMENT__DEPLOY_EXT_GROUP:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), true, false));
				return;
		}
		super.notifyChanged(notification);
	}

	/**
	 * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children
	 * that can be created under this object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object)
	{
		super.collectNewChildDescriptors(newChildDescriptors, object);

		newChildDescriptors.add
			(createChildParameter
				(DeploymentPackage.Literals.DEPLOYMENT__BINDINGS,
				 DeploymentFactory.eINSTANCE.createBindings()));

		newChildDescriptors.add
			(createChildParameter
				(DeploymentPackage.Literals.DEPLOYMENT__SERVICES,
				 DeploymentFactory.eINSTANCE.createServices()));

		newChildDescriptors.add
			(createChildParameter
				(DeploymentPackage.Literals.DEPLOYMENT__RUIAPPLICATION,
				 DeploymentFactory.eINSTANCE.createRUIApplication()));

		newChildDescriptors.add
			(createChildParameter
				(DeploymentPackage.Literals.DEPLOYMENT__RESOURCE_OMISSIONS,
				 DeploymentFactory.eINSTANCE.createResourceOmissions()));

		newChildDescriptors.add
			(createChildParameter
				(DeploymentPackage.Literals.DEPLOYMENT__TARGET_GROUP,
				 FeatureMapUtil.createEntry
					(DeploymentPackage.Literals.EGL_DEPLOYMENT_ROOT__TARGET_PROJECT,
					 DeploymentFactory.eINSTANCE.createDeploymentProject())));

		newChildDescriptors.add
			(createChildParameter
				(DeploymentPackage.Literals.DEPLOYMENT__INCLUDE,
				 DeploymentFactory.eINSTANCE.createInclude()));
	}

	/**
	 * Return the resource locator for this item provider's resources.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public ResourceLocator getResourceLocator()
	{
		return EglddEditPlugin.INSTANCE;
	}

}
